@@ -1109,8 +1109,15 @@ void pool_add_dml_table_oid(int oid)
 
 	if (oidbufp >= oidbuf_size)
 	{
+		MemoryContext oldcxt;
 		oidbuf_size += POOL_OIDBUF_SIZE;
+		/*
+		 * This need to live throughout the life of child so home it in
+		 * TopMemoryContext
+		 */
+		oldcxt = MemoryContextSwitchTo(TopMemoryContext);
 		tmp = repalloc(oidbuf, sizeof(int) * oidbuf_size);
+		oldcxt = MemoryContextSwitchTo(oldcxt);
 		if (tmp == NULL)
 			return;
 
@@ -1120,12 +1127,13 @@ void pool_add_dml_table_oid(int oid)
 	for (i=0;i<oidbufp;i++)
 	{
 		if (oidbuf[i] == oid)
-			/* Already same oid exists */
+		/* Already same oid exists */
 			return;
 	}
 	oidbuf[oidbufp++] = oid;
 }
 
+
 /*
  * Get table oid buffer
  */
@@ -1730,7 +1738,7 @@ static int pool_get_memqcache_blocks(void)
  */
 size_t pool_shared_memory_cache_size(void)
 {
-	int num_blocks;
+	int64 num_blocks;
 	size_t size;
 
 	if (pool_config->memqcache_maxcache > pool_config->memqcache_cache_block_size)
@@ -1746,13 +1754,13 @@ size_t pool_shared_memory_cache_size(void)
 	if (num_blocks == 0)
 		ereport(FATAL,
 			(errmsg("invalid memory cache configuration"),
-					errdetail("memqcache_total_size %d should be greater or equal to memqcache_cache_block_size %d",
+					errdetail("memqcache_total_size %ld should be greater or equal to memqcache_cache_block_size %d",
 								pool_config->memqcache_total_size,
 								pool_config->memqcache_cache_block_size)));
 
 		ereport(LOG,
 			(errmsg("memory cache initialized"),
-					errdetail("memcache blocks :%d",num_blocks)));
+					errdetail("memcache blocks :%ld",num_blocks)));
 	/* Remember # of blocks */
 	pool_set_memqcache_blocks(num_blocks);
 	size = pool_config->memqcache_cache_block_size * num_blocks;
@@ -3146,6 +3154,24 @@ void pool_handle_query_cache(POOL_CONNECTION_POOL *backend, char *query, Node *n
 								(errmsg("ReadyForQuery: pool_commit_cache failed")));
 
 					}
+					else
+					{
+						/*
+						 * Reset temporary query cache buffer. This is
+						 * necessary if extended query protocol is used and a
+						 * bind/execute message arrives which uses a statement
+						 * created by prior parse message. In this case since
+						 * the temp_cache is not initialized by a parse
+						 * message, messages are added to pre existing temp
+						 * cache buffer. The problem was found in bug#152.
+						 * http://www.pgpool.net/mantisbt/view.php?id=152
+						 */
+						POOL_TEMP_QUERY_CACHE *cache;
+						cache = pool_get_current_cache();
+						pool_discard_temp_query_cache(cache);
+						session_context->query_context->temp_cache = pool_create_temp_query_cache(query);
+					}
+
 					pfree(cache_buffer);
 				}
 				pool_shmem_unlock();
